{ TDSPanel is an enhanced Panel component that allows you to display a
  drop-shadow. The component is very easy to use, and possesses only four
  extra properties:

  PanelType        - describes the type of panel displayed, and can be either
                     drop-shadow or plain;
  ShadowColor      - the color of the dropped shadow;
  XOffSet          - offset in the X direction;
  YOffSet          - offset in the Y direction; }

{ Written by N.Hollinworth 2nd May 1999 }

unit DSPanel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

const
  OffSet = 5;

type
  TPanelType = (Plain, DropShadow);

  TDSPanel = class(TPanel)
  private
    FAlignment: TAlignment;
    FShadowColor: TColor;
    FPanelType: TPanelType;
    FXOffSet: integer;
    FYOffSet: integer;
    procedure Paint; override;
    procedure SetShadowColor(const Value: TColor);
    procedure SetAlignment(const Value: TAlignment);
    procedure SetPanelType(const Value: TPanelType);
    procedure SetXOffSet(const Value: integer);
    procedure SetYOffSet(const Value: integer);
    procedure CMParentColorChanged(var Message: TMessage); message CM_PARENTCOLORCHANGED;
  protected
    property Alignment: TAlignment read FAlignment write SetAlignment default taCenter;
  public
    constructor Create (AOwner: TComponent); override;
  published
    property PanelType: TPanelType read FPanelType write SetPanelType default Plain;
    property XOffSet: integer read FXOffSet write SetXOffSet default OffSet;
    property YOffSet: integer read FYOffSet write SetYOffSet default OffSet;
    property ShadowColor: TColor read FShadowColor write SetShadowColor;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TDSPanel]);
end;

{ TDSPanel }

constructor TDSPanel.Create(AOwner: TComponent);
begin
  inherited;
  FXOffSet := OffSet;
  FYOffSet := OffSet;
  FShadowColor := clBlack;
  FAlignment := taCenter;
  FPanelType := Plain;
end;

procedure TDSPanel.Paint;
const
  Alignments: array[TAlignment] of Longint = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  Rect: TRect;
  TopColor, BottomColor: TColor;
  FontHeight: Integer;
  Flags: Longint;
  Shadow: TRect;
  Face: TRect;

  procedure AdjustColors(Bevel: TPanelBevel);
  begin
    TopColor := clBtnHighlight;
    if Bevel = bvLowered then TopColor := clBtnShadow;
    BottomColor := clBtnShadow;
    if Bevel = bvLowered then BottomColor := clBtnHighlight;
  end;

begin
  Rect := GetClientRect;

  Canvas.Brush.Color := Parent.Brush.Color;
  Canvas.FillRect(Rect);

  case FPanelType of
  Plain:
  begin
    Face.Left   := Rect.Left;
    Face.Top    := Rect.Top;
    Face.Right  := Rect.Right;
    Face.Bottom := Rect.Bottom;
    Canvas.Brush.Color := Color;
    Canvas.FillRect(Face);
  end;
  DropShadow:
  begin
    // Set coordinates for drop shadow
    Shadow.Left   := Rect.Left + FXOffSet;
    Shadow.Top    := Rect.Top + FYOffSet;
    Shadow.Right  := Rect.Left + FXOffSet + Width;
    Shadow.Bottom := Rect.Top + FYOffSet + Height;
    Canvas.Brush.Color := FShadowColor;
    Canvas.FillRect(Shadow);

    // Set coordinates for face
    if FXOffSet < 0 then begin
      Face.Left := Rect.Left - FXOffSet;
      Face.Right := Rect.Right;
    end
    else begin
      Face.Left   := Rect.Left;
      Face.Right  := Rect.Right - FXOffSet;
    end;

    if FYOffSet < 0 then begin
      Face.Top := Rect.Top - FYOffSet;
      Face.Bottom := Rect.Bottom;
    end
    else begin
      Face.Top    := Rect.Top;
      Face.Bottom := Rect.Bottom - FYOffSet;
    end;
    
    Canvas.Brush.Color := Color;
    Canvas.FillRect(Face);
  end;
  end; { case }

  if BevelOuter <> bvNone then
  begin
    AdjustColors(BevelOuter);
    Frame3D(Canvas, Face, TopColor, BottomColor, BevelWidth);
  end;
  Frame3D(Canvas, Face, Color, Color, BorderWidth);
  if BevelInner <> bvNone then
  begin
    AdjustColors(BevelInner);
    Frame3D(Canvas, Face, TopColor, BottomColor, BevelWidth);
  end;

  with Canvas do
  begin
    Brush.Style := bsClear;
    Font := Self.Font;
    FontHeight := TextHeight('W');
    with Face do
    begin
      Top := ((Bottom + Top) - FontHeight) div 2;
      Bottom := Top + FontHeight;
    end;
    Flags := DT_EXPANDTABS or DT_VCENTER or Alignments[FAlignment];
    Flags := DrawTextBiDiModeFlags(Flags);
    DrawText(Handle, PChar(Caption), -1, Face, Flags);
    ValidateRect(Handle, nil);
  end;
end;

procedure TDSPanel.SetAlignment(const Value: TAlignment);
begin
  FAlignment := Value;
  Invalidate;
end;

procedure TDSPanel.SetShadowColor(const Value: TColor);
begin
  FShadowColor := Value;
  Invalidate;
end;

procedure TDSPanel.SetPanelType(const Value: TPanelType);
begin
  FPanelType := Value;
  Invalidate;
end;

procedure TDSPanel.SetXOffSet(const Value: integer);
begin
  FXOffSet := Value;
  Invalidate;
end;

procedure TDSPanel.SetYOffSet(const Value: integer);
begin
  FYOffSet := Value;
  Invalidate;
end;

procedure TDSPanel.CMParentColorChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

end.
